package com.jzlife.report.util;

import com.jzlife.report.anno.ExcelFiled;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.eventusermodel.XSSFReader;
import org.apache.poi.xssf.model.SharedStringsTable;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;


@Slf4j
public abstract class ExcelPOIUtils extends DefaultHandler {
    final static String errormsg = "您好,第 [%s]行,第[%s]列%s,请重新输入!";
    private final static String xls = "xls";
    private final static String xlsx = "xlsx";
    private SharedStringsTable sst;
    private String lastContents;
    private boolean nextIsString;
    private final List<String> rowlist = new ArrayList<>();
    private int curRow = 0; //当前行
    private int curCol = 0; //当前列索引
    private int preCol = 0; //上一列列索引
    private int titleRow = 0; //标题行，一般情况下为0
    private int rowsize = 0; //列数
    private int sheetIndex = -1;

    static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public ExcelPOIUtils() {

    }

    SimpleDateFormat createDateFormat(String pattern) {
        return new SimpleDateFormat(pattern);
    }

    public static Workbook getWorkBook(MultipartFile file) throws Exception {
        // 获得文件名
        String fileName = file.getOriginalFilename();
        // 创建Workbook工作薄对象，表示整个excel
        Workbook workbook = null;
        try {
            // 获取excel文件的io流
            InputStream is = file.getInputStream();
            // 根据文件后缀名不同(xls和xlsx)获得不同的Workbook实现类对象
            if (Objects.requireNonNull(fileName).endsWith(xls)) {
                // 2003
                workbook = new HSSFWorkbook(is);
            } else if (fileName.endsWith(xlsx)) {
                // 2007
                workbook = new XSSFWorkbook(is);
            }
        } catch (IOException e) {
            throw new Exception("解析报文出错!");
        }
        return workbook;
    }


    public void processOneSheet(String filename) throws Exception {
        OPCPackage pkg = OPCPackage.open(filename);
        XSSFReader r = new XSSFReader(pkg);
        SharedStringsTable sst = r.getSharedStringsTable();

        XMLReader parser = fetchSheetParser(sst);

        // 获得第一个sheet
        InputStream sheet1 = r.getSheet("rId1");
        InputSource sheetSource = new InputSource(sheet1);
        parser.parse(sheetSource);
        sheet1.close();
    }

    public XMLReader fetchSheetParser(SharedStringsTable sst) throws SAXException {
        XMLReader parser = XMLReaderFactory.createXMLReader();
        this.sst = sst;
        parser.setContentHandler(this);
        return parser;
    }


    public ExcelPOIUtils(SharedStringsTable sst) {
        this.sst = sst;
    }

    public void startElement(String uri, String localName, String name,
                             Attributes attributes) {
        // c => 单元格
        if (name.equals("c")) {
            // 如果下一个元素是 SST 的索引，则将nextIsString标记为true
            String cellType = attributes.getValue("t");
            String rowStr = attributes.getValue("r");
            curCol = this.getRowIndex(rowStr);
            nextIsString = cellType != null && cellType.equals("s");
        }
        // 置空
        lastContents = "";
    }

    public void endElement(String uri, String localName, String name) {
        // 根据SST的索引值的到单元格的真正要存储的字符串
        // 这时characters()方法可能会被调用多次
        if (nextIsString) {
            try {
                int idx = Integer.parseInt(lastContents);
                lastContents = new XSSFRichTextString(sst.getEntryAt(idx))
                        .toString();
            } catch (Exception ignored) {
            }
        }
        // v => 单元格的值，如果单元格是字符串则v标签的值为该字符串在SST中的索引
        // 将单元格内容加入rowlist中，在这之前先去掉字符串前后的空白符
        if (name.equals("v")) {
            String value = lastContents.trim();
            value = value.equals("") ? " " : value;
            int cols = curCol - preCol;
            if (cols > 1) {
                for (int i = 0; i < cols - 1; i++) {
                    rowlist.add(preCol, "");
                }
            }
            preCol = curCol;
            rowlist.add(curCol - 1, value);
        } else {
            //如果标签名称为 row ，这说明已到行尾，调用 optRows() 方法
            if (name.equals("row")) {
                int tmpCols = rowlist.size();
                if (curRow > this.titleRow && tmpCols < this.rowsize) {
                    for (int i = 0; i < this.rowsize - tmpCols; i++) {
                        rowlist.add(rowlist.size(), "");
                    }
                }
                try {
                    optRows(sheetIndex, curRow, rowlist);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                if (curRow == this.titleRow) {
                    this.rowsize = rowlist.size();
                }
                rowlist.clear();
                curRow++;
                curCol = 0;
                preCol = 0;
            }
        }
    }

    public abstract void optRows(int sheetIndex, int curRow, List<String> rowlist) throws IllegalAccessException;

    //读取元素间内容时的handler
    public void characters(char[] ch, int start, int length) {
        lastContents += new String(ch, start, length);
    }

    public int getRowIndex(String rowStr) {
        rowStr = rowStr.replaceAll("[^A-Z]", "");
        byte[] rowAbc = rowStr.getBytes();
        int len = rowAbc.length;
        float num = 0;
        for (int i = 0; i < len; i++) {
            num += (rowAbc[i] - 'A' + 1) * Math.pow(26, len - i - 1);
        }
        return (int) num;
    }

    public int getTitleRow() {
        return titleRow;
    }

    public void setTitleRow(int titleRow) {
        this.titleRow = titleRow;
    }


    /**
     * @param file     文件
     * @param class1   实体类
     * @param errorMap 错误map   key 行号   value  具体报错信息
     * @param errors   错误列表
     * @param colCount 列数
     * @param i
     * @return
     * @throws Exception
     */
    public static <T> List<T> readExc(MultipartFile file, Class<T> class1, Map<Integer, String> errorMap, List<T> errors, int colCount, int i) throws Exception {
        try {
            if (i != 1) {
                Workbook workbook = getWorkBook(file);
                return readExcel(class1, errorMap, errors, workbook, colCount);
            }
            return readBigExcel(class1, file);
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }

    }

    private static <T> List<T> readBigExcel(Class<T> class1, MultipartFile file) {
        File convFile = new File(System.getProperty("java.io.tmpdir") + "/" + file.getOriginalFilename());
        List<T> list = new ArrayList<>();
        ExcelPOIUtils utils = new ExcelPOIUtils() {
            @Override
            public void optRows(int sheetIndex, int curRow, List<String> rowlist) throws IllegalAccessException {
                if (curRow == 0) {
                    return;
                }
                T obj = null;
                try {
                    obj = class1.newInstance();
                } catch (InstantiationException | IllegalAccessException e) {
                    e.printStackTrace();
                }
                Field[] fields = class1.getDeclaredFields();
                for (int i = 0; i < fields.length; i++) {
                    fields[i].setAccessible(true);
                    if (fields[i].getType().equals(Integer.class)) {
                        fields[i].set(obj, Integer.parseInt(rowlist.get(i)));
                    } else {
                        fields[i].set(obj, rowlist.get(i));
                    }
                }
                System.out.println(obj);
                list.add(obj);
            }
        };
        try {
            file.transferTo(convFile);
            utils.processOneSheet(convFile.getAbsolutePath());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return list;
    }

    public static <T> List<T> readExc(InputStream in, Class<T> class1, Map<Integer, String> errorMap, List<T> errors, int colCount) throws Exception {
        try {
            Workbook workbook = WorkbookFactory.create(in);
            return readExcel(class1, errorMap, errors, workbook, colCount);
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception(e.getMessage());
        }
    }

    private static <T> List<T> readExcel(Class<T> class1, Map<Integer, String> errorMap, List<T> errors, Workbook workbook, int colCount)
            throws Exception {
        Sheet sheet = workbook.getSheetAt(0);
        int rowCount = sheet.getPhysicalNumberOfRows(); // 获取总行数
        if (rowCount == 1) { // 表格最小长度应为2
            return null;
        }
        int coloumNum = sheet.getRow(2).getPhysicalNumberOfCells();
        if (coloumNum != colCount) {
            throw new Exception("excel格式跟模板格式不一致!");
        }

        if (rowCount < 10000) {
            return singleRead(class1, errorMap, errors, sheet, rowCount, 1);
        } else {
            int count = rowCount / 10000 + 1;
            ExecutorService executorService = Executors.newFixedThreadPool(count);
            List<T> list = new ArrayList<>();
            for (int i = 0; i < count; i++) {
                int finalI = i + 1;
                int endSize = 10000 * finalI;
                int startSize = 10000 * i + 1;
                Callable callable = () -> singleRead(class1, errorMap, errors, sheet, endSize, startSize);
                Future<T> future = executorService.submit(callable);
                T t = future.get();
                list.add(t);
            }
            executorService.shutdown();
            return list;
        }

    }

    private static <T> List<T> singleRead(Class<T> class1, Map<Integer, String> errorMap, List<T> errors, Sheet sheet, int rowCount, int start) throws InstantiationException, IllegalAccessException {
        List<T> list = new ArrayList<>(rowCount - 1);
        T obj;
        // 遍历每一行
        for (int r = start; r < rowCount; r++) {
            Row row = sheet.getRow(r);
            obj = class1.newInstance();
            Field[] fields = class1.getDeclaredFields();
            Field field;
            boolean flag = true;     //标识,确定该条数据是否通过第一轮判断
            boolean errorFlag = false;  //标识是否有错误点
            StringBuilder error = new StringBuilder();
            for (Field value : fields) {
                field = value;
                ExcelFiled excelFiled = field.getAnnotation(ExcelFiled.class);
                if (excelFiled == null) {
                    continue;
                }
                Cell cell = row.getCell(excelFiled.colIndex());
                try {
                    //序号为0,编号为空,则为无效行
                    if (excelFiled.colIndex() == 0) {
                        if (cell == null || cell.toString().length() == 0) {
                            flag = false;
                            break;
                        }
                    }

                    if (!excelFiled.skip()) {   //必填字段,需要判断是否非空
                        if (cell == null || cell.toString().length() == 0) {
                            //errogMsg(errorMap, r, j, excelFiled,"不能为空");
                            error.append(excelFiled.colName()).append("不能为空").append("|");
                            flag = false;
                            errorFlag = true;
                            continue;
                        }
                    }
                    if (field.getType().equals(Date.class)) {
                        if (Cell.CELL_TYPE_STRING == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), dateFormat.parse(cell.getStringCellValue()));
                        } else {
                            ReflectUtil.setValue(obj, field.getName(), new Date(cell.getDateCellValue().getTime()));
                        }
                    } else if (field.getType().equals(Integer.class)) {
                        if (Cell.CELL_TYPE_NUMERIC == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), (int) cell.getNumericCellValue());
                        } else if (Cell.CELL_TYPE_STRING == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), Integer.parseInt(cell.getStringCellValue()));
                        }
                    } else if (field.getType().equals(BigDecimal.class)) {
                        if (Cell.CELL_TYPE_NUMERIC == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), BigDecimal.valueOf(cell.getNumericCellValue()));
                        } else if (Cell.CELL_TYPE_STRING == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), cell.getStringCellValue());
                        }
                    } else if (field.getType().equals(Double.class)) {
                        if (Cell.CELL_TYPE_NUMERIC == cell.getCellType()) {
                            if (excelFiled.precision() == 0) {   //没有小数点
                                ReflectUtil.setValue(obj, field.getName(), (double) BigDecimal.valueOf(cell.getNumericCellValue()).intValue());
                            } else {
                                ReflectUtil.setValue(obj, field.getName(), BigDecimal.valueOf(cell.getNumericCellValue()).doubleValue());
                            }
                        } else if (Cell.CELL_TYPE_STRING == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), new Double(cell.getStringCellValue()));
                        }
                    } else if (field.getType().equals(String.class)) {
                        if (Cell.CELL_TYPE_NUMERIC == cell.getCellType()) {
                            if (excelFiled.precision() == 0) {   //没有小数点
                                ReflectUtil.setValue(obj, field.getName(), BigDecimal.valueOf(cell.getNumericCellValue()).intValue() + "");
                            } else {
                                ReflectUtil.setValue(obj, field.getName(), BigDecimal.valueOf(cell.getNumericCellValue()).doubleValue() + "");
                            }
                        } else if (Cell.CELL_TYPE_STRING == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), cell.getStringCellValue());
                        }
                    } else {
                        if (Cell.CELL_TYPE_NUMERIC == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), BigDecimal.valueOf(cell.getNumericCellValue()));
                        } else if (Cell.CELL_TYPE_STRING == cell.getCellType()) {
                            ReflectUtil.setValue(obj, field.getName(), cell.getStringCellValue());
                        }
                    }
                } catch (Exception e) {
                    flag = false;
                    errorFlag = true;
                    error.append(excelFiled.colName()).append("类型格式有误").append("|");
                }
            }
            if (flag) {
                list.add(obj);
            }
            if (errorFlag) {
                errorMap.put(r, error.toString());
                errors.add(obj);
            }
        }
        return list;
    }


}
